Release 980301

Sun Mar  1 10:45:23 1998  Andreas Mohr <100.30936@germany.net>

	* [loader/ne_image.c]
	Fixed problem with weird DLLs (NE_FFLAGS_SINGLEDATA && DGROUP = 0).

	* [msdos/dosmem.c]
	Export address for __0000H, too.

	* [msdos/dpmi.c]
	Changed MemAlloc functions to return less fragmented addresses.

Sat Feb 28 18:50:12 1998  Alexandre Julliard  <julliard@lrc.epfl.ch>

	* [scheduler/process.c] [scheduler/sysdeps.c]
	Don't use %fs register before threading initialization.

Sat Feb 28 14:04:56 1998  Kristian Nielsen  <kristian.nielsen@risoe.dk>

	* [configure.in] [include/acconfig.h]
	Autoconf macro to check for non-reentrant X libraries.

	* [windows/winpos.c]
	In SetWindowPos32(), do not cause WM_SIZE messages when the
	SWP_NOSIZE flag is specified. This fixes the division-by-zero in
	Borland C++ 4.0 "Open Project" menu item.

Sat Feb 28 13:11:26 1998  James Moody  <013263m@dragon.acadiau.ca>

	* [ole/ole2nls.c]
	Changed "English" values from German to English.

	* [files/dos_fs.c]
	Fixed off-by-one month bug.

Fri Feb 27 22:12:01 1998  Douglas Ridgway  <ridgway@winehq.com>

	* [windows/win.c]
	Fix winelib class menu loading bug.

	* [include/module.h] [loader/module.c]
	LoadModule32 should be implemented in terms of CreateProcess.

	* [programs/view/*]
	Metafile viewer sample program.

	* [documentation/wine.texinfo] [documentation/Makefile.in]
	Improvements and additions, HTML target.

Fri Feb 27 04:27:48 1998  Dimitrie O. Paun <dimi@cs.toronto.edu>

	* [*/*]
	Switched to the new debug messages interface. For more information
	please refer to documentation/debug-msgs. Because the new scheme
	introduces a new semantic level, I had to manually do through
	about 530 dprintf_xxx!  The rest of about 2400 where transformed
	via a script.  Because of the large number of changes that I had
	to do, some may have not come out as nicely as I wanted them.  If
	this is the case, please let me know. There is a lot of work left
	to do: -- a few hundred printf's to be converted -- about 2300
	fprintf's to be converted -- about 600 FIXME's to be transformed
	The problem is that in the above mentioned cases, a lot of manual
	intervention is required because a lot of the information is
	missing.  There are also a lot of other things to be done to the
	interface and so forth. I have now ideas for a at least a month
	worth of full time work :) I will proceed with many changes in the
	next few releases, so please do not start modifing things because
	there will be a hell of a lot of conflicts. If you have ideas that
	you want to integrate or you want to work on different things,
	please coordinate with me.

Thu Feb 26 13:04:29 1998  David Lee Lambert <lamber45@egr.msu.edu>

	* [ole/ole2nls.c] [include/windows.h]
	First try at OLE date- and time-formatting functions.

Wed Feb 25 11:20:35 1998 Marcus Meissner <msmeissn@cip.informatik.uni-erlangen.de>

	* [files/*.c]
	Changed dos device handling, added 'CON' devicehandling.

	* [graphics/ddraw.c]
	Bug fixes, some additions.

	* [if1632/builtin.c][loader/module.c][library/winestub.c]
	Small hack so we don't need a dummy BUILTIN_LoadModule
	in winestub.c.

	* [ole/*][relay32/ole32.spec][if1632/storage.spec]
	storage.dll started. winword loads documents (saving
	doesn't work yet, dunno why).
	Several ole additions, some cleanups and bugfixes.
	IMalloc16 implemented.

	* [loader/pe_image.c]
	Added some comments, fixed circular dll references,
	fixed modref ordering, fixed tls allocation.

	* [memory/global.c]
	Added validity checks before every GET_ARENA_PTR.
	(several functions rely on Global* return values
	 on invalid handles, like IsTask).
	Implemented GlobalUnlockFree16.

	* [memory/virtual.c]
	Replaced dprintf_virtual by fprintf, so we can
	do 'info map' again in the debugger. Increase read
	linesize for Linux2.1 cases.

	* [misc/cpu.c][misc/registry.c]
	Moved cpu registry initialization to misc/cpu.c.

	* [multimedia/dsound.c]
	Enhanced, replaced GETOSPACE bufferingcheck by SETFRAGMENT.

	* [relay32/crtdll.spec][relay32/ntdll.spec]
	Replaced some ptr by respective 'str' and 'wstr' arguments
	for libc functions.

	* [scheduler/thread.c]
	Added some sanity checks to stackallocation, tlshandling fixed.

	* [tools/build.c]
	Fixed cdecl argumenttype order (was reversed).

	* [win32/ordinals.c]
	Implemented KERNEL_449.

	* [windows/dinput.c]
	Some fixes, needs much more work. Tomb Raider2 works with keyboard ;)

Tue Feb 24 20:46:37 1998  James Juran   <jrj120@psu.edu>

	* [windows/win.c]
	Fixed USER32 ordinal numbers in documentation.

Sat Feb 21 12:30:38 1998  John Richardson <jrichard@zko.dec.com>

	* [files/file.c] [include/k32obj.h] [memory/virtual.c]
	[scheduler/critsection.c] [scheduler/event.c] [scheduler/handle.c]
	[scheduler/k32obj.c] [scheduler/mutex.c] [scheduler/process.c]
	[scheduler/semaphore.c] [scheduler/thread.c]
	Added generic k32obj read and write routines for k32objs that
	support I/O.

	* [documentation/console]
	Updated console docs.

	* [win32/console.c]
	Make console work like a k32obj that supports I/O.

	* [include/windows.h]
	Make WriteFile and ReadFile take HANDLE32 for handle.

Sun Feb 15 14:07:07 1998  Dimitrie O. Paun  <dimi@mail.cs.toronto.edu>

	* [controls/menu.c] [misc/ver.c] [multimedia/dsound.c]
	  [multimedia/joystick.c] [windows/dialog.c]
	Modified some dprintf_xxx's to prepare them for a new
	dprintf_ scheme. Basically, I changed the dprintf's that
	outputed a line with many dprintf calls to do just one
	dprintf call.
diff --git a/win32/console.c b/win32/console.c
index 2396f34..d2f13b5 100644
--- a/win32/console.c
+++ b/win32/console.c
@@ -9,7 +9,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
-#include <pty.h>
 #include <termios.h>
 #include <sys/ioctl.h>
 #include <sys/types.h>
@@ -25,7 +24,6 @@
 #include "winerror.h"
 #include "wincon.h"
 #include "heap.h"
-#include "stddebug.h"
 #include "debug.h"
 
 static CONSOLE_SCREEN_BUFFER_INFO dummyinfo =
@@ -39,23 +37,26 @@
 
 /* The console -- I chose to keep the master and slave
  * (UNIX) file descriptors around in case they are needed for
  * ioctls later.  The pid is needed to detroy the xterm if needed.
  */
 typedef struct _CONSOLE {
 	K32OBJ  header;
 	int	master;			/* xterm side of pty */
 	int	slave;			/* wine side of pty */
 	int	pid;			/* xterm's pid, -1 if no xterm */
-	int	flags;			/* CONSOLE_STARTED_FROM */
-        K32OBJ *file_in;                /* console input */
-        K32OBJ *file_out;               /* console output */
-        K32OBJ *file_err;               /* console error */
 } CONSOLE;
 
-#define CONSOLE_STARTED_FROM  (0x1)	/* FIXME: this is lame, it should have 
-					   something to do with sharing... */
 
 static void CONSOLE_Destroy( K32OBJ *obj );
+static BOOL32 CONSOLE_Write(K32OBJ *ptr, LPCVOID lpBuffer, 
+			    DWORD nNumberOfChars,  LPDWORD lpNumberOfChars, 
+			    LPOVERLAPPED lpOverlapped);
+static BOOL32 CONSOLE_Read(K32OBJ *ptr, LPVOID lpBuffer, 
+			   DWORD nNumberOfChars, LPDWORD lpNumberOfChars, 
+			   LPOVERLAPPED lpOverlapped);
+static int wine_openpty(int *master, int *slave, char *name, 
+		     struct termios *term, struct winsize *winsize);
+static BOOL32 wine_createConsole(int *master, int *slave, int *pid);
 
 const K32OBJ_OPS CONSOLE_Ops =
 {
@@ -63,13 +64,12 @@
 	NULL,			/* satisfied */
 	NULL,			/* add_wait */
 	NULL,			/* remove_wait */
+	CONSOLE_Read,		/* read */
+	CONSOLE_Write,		/* write */
 	CONSOLE_Destroy		/* destroy */
 };
 
 
-static int wine_openpty(int *master, int *slave, char *name, 
-		     struct termios *term, struct winsize *winsize);
-
 
 
 static void CONSOLE_Destroy(K32OBJ *obj)
@@ -84,10 +84,55 @@
 		kill(console->pid, SIGTERM);
 	}
 	HeapFree(SystemHeap, 0, console);
-
 }
 
 
+/* lpOverlapped is ignored */
+static BOOL32 CONSOLE_Read(K32OBJ *ptr, LPVOID lpBuffer, DWORD nNumberOfChars,
+			LPDWORD lpNumberOfChars, LPOVERLAPPED lpOverlapped)
+{
+	CONSOLE *console = (CONSOLE *)ptr;
+	int result;
+
+	dprintf_info(console, "CONSOLE_Read: %p %p %ld\n", ptr, lpBuffer,
+		      nNumberOfChars);
+
+	*lpNumberOfChars = 0; 
+
+	if ((result = read(console->slave, lpBuffer, nNumberOfChars)) == -1) {
+		FILE_SetDosError();
+		return FALSE;
+	}
+	*lpNumberOfChars = result;
+	return TRUE;
+}
+
+/* lpOverlapped is ignored */
+static BOOL32 CONSOLE_Write(K32OBJ *ptr, LPCVOID lpBuffer, 
+			    DWORD nNumberOfChars,
+			    LPDWORD lpNumberOfChars, LPOVERLAPPED lpOverlapped)
+{
+	CONSOLE *console = (CONSOLE *)ptr;
+	int result;
+
+	dprintf_info(console, "CONSOLE_Write: %p %p %ld\n", ptr, lpBuffer,
+		      nNumberOfChars);
+
+	*lpNumberOfChars = 0; 
+
+	/* FIXME: there was a loop here before, why??? */
+
+	if ((result = write(console->slave, lpBuffer, nNumberOfChars)) == -1) {
+		FILE_SetDosError();
+		return FALSE;
+	}
+	*lpNumberOfChars = result;
+	return TRUE;
+}
+
+
+
+
 /***********************************************************************
  *           SetConsoleCtrlHandler               (KERNEL32.459)
  */
@@ -164,9 +209,8 @@
 		return FALSE;
 	}
 
-        if (console->file_in) K32OBJ_DecCount( console->file_in );
-        if (console->file_out) K32OBJ_DecCount( console->file_out );
-        if (console->file_err) K32OBJ_DecCount( console->file_err );
+	PROCESS_CloseObjHandles(pdb, &console->header);
+
 	K32OBJ_DecCount( &console->header );
 	pdb->console = NULL;
 	SYSTEM_UNLOCK();
@@ -264,6 +308,24 @@
 
 }
 
+/***********************************************************************
+ *		CONSOLE_GetConsoleHandle
+ *	 returns a 16-bit style console handle
+ * note: only called from _lopen
+ */
+HFILE32 CONSOLE_GetConsoleHandle(VOID)
+{
+	PDB32 *pdb = PROCESS_Current();
+	HFILE32 handle = HFILE_ERROR32;
+
+	SYSTEM_LOCK();
+	if (pdb->console != NULL) {
+		CONSOLE *console = (CONSOLE *)pdb->console;
+		handle = (HFILE32)HANDLE_Alloc(&console->header, 0, TRUE);
+	}
+	SYSTEM_UNLOCK();
+	return handle;
+}
 
 /***********************************************************************
  *            AllocConsole (KERNEL32.103)
@@ -287,13 +349,8 @@
 
 	console = (CONSOLE *)pdb->console;
 
-	/* we only want to be able to open a console if the process doesn't have one
-	 * now or we got the one we have from our parent
-	 *  - invalid handle comes from when the console was closed via FreeConsole()
-	 *  - CONSOLE_STARTED_FROM is when this process inherits its console from
-	 *    its parent
-	 */
-	if (console && (console->flags & CONSOLE_STARTED_FROM) == 0) {
+	/* don't create a console if we already have one */
+	if (console != NULL) {
 		SetLastError(ERROR_ACCESS_DENIED);
 		SYSTEM_UNLOCK();
 		return FALSE;
@@ -308,9 +365,6 @@
         console->header.type     = K32OBJ_CONSOLE;
         console->header.refcount = 1;
         console->pid             = -1;
-        console->file_in         = NULL;
-        console->file_out        = NULL;
-        console->file_err        = NULL;
 
 	if (wine_createConsole(&master, &slave, &pid) == FALSE) {
 		K32OBJ_DecCount(&console->header);
@@ -322,26 +376,24 @@
         console->master = master;
 	console->slave = slave;
 	console->pid = pid;
-	console->flags = 0;
 
-	if ((hIn = FILE_DupUnixHandle(slave)) == INVALID_HANDLE_VALUE32)
+	if ((hIn = HANDLE_Alloc(&console->header, 0, TRUE)) == INVALID_HANDLE_VALUE32)
         {
             K32OBJ_DecCount(&console->header);
             SYSTEM_UNLOCK();
             return FALSE;
 	}
-	FILE_SetFileType(hIn, FILE_TYPE_CHAR);
 
-	if ((hOut = FILE_DupUnixHandle(slave)) == INVALID_HANDLE_VALUE32)
+	if ((hOut = HANDLE_Alloc(&console->header, 0, TRUE)) == INVALID_HANDLE_VALUE32)
         {
             CloseHandle(hIn);
             K32OBJ_DecCount(&console->header);
             SYSTEM_UNLOCK();
             return FALSE;
 	}
-	FILE_SetFileType(hOut, FILE_TYPE_CHAR);
 
-	if ((hErr = FILE_DupUnixHandle(slave)) == INVALID_HANDLE_VALUE32)
+
+	if ((hErr = HANDLE_Alloc(&console->header, 0, TRUE)) == INVALID_HANDLE_VALUE32)
         {
             CloseHandle(hIn);
             CloseHandle(hOut);
@@ -349,11 +401,6 @@
             SYSTEM_UNLOCK();
             return FALSE;
 	}
-	FILE_SetFileType(hErr, FILE_TYPE_CHAR);
-
-        console->file_in = HANDLE_GetObjPtr( hIn, K32OBJ_FILE, 0 /*FIXME*/ );
-        console->file_out = HANDLE_GetObjPtr( hIn, K32OBJ_FILE, 0 /*FIXME*/ );
-        console->file_err = HANDLE_GetObjPtr( hIn, K32OBJ_FILE, 0 /*FIXME*/ );
 
 	/* associate this console with the process */
         if (pdb->console) K32OBJ_DecCount( pdb->console );
@@ -436,18 +483,6 @@
 	/* FIXME: should I check if this is a console handle? */
 	return WriteFile(hConsoleOutput, lpBuffer, nNumberOfCharsToWrite,
 			 lpNumberOfCharsWritten, NULL);
-
-#ifdef OLD
-	*lpNumberOfCharsWritten = fprintf(CONSOLE_console.conIO, "%.*s",
-					   (int)nNumberOfCharsToWrite,
-					   (LPSTR)lpBuffer );
-	if (ferror(CONSOLE_console.conIO) {
-		clearerr();
-		return FALSE;
-	}
-
-	return TRUE;
-#endif
 }
 
 /***********************************************************************
@@ -476,20 +511,6 @@
 	return WriteFile(hConsoleOutput, lpBuffer, nNumberOfCharsToWrite,
 			 lpNumberOfCharsWritten, NULL);
 
-
-#ifdef OLD
-	LPSTR buf =  HEAP_strdupWtoA( GetProcessHeap(), 0, lpBuffer );
-	*lpNumberOfCharsWritten = fprintf(CONSOLE_console.conIO, "%.*s",
-					  (int)nNumberOfCharsToWrite, buf );
-	HeapFree( GetProcessHeap(), 0, buf );
-
-	if (ferror(CONSOLE_console.conIO) {
-		clearerr();
-		return FALSE;
-	}
-
-	return TRUE;
-#endif
 }
 
 /***********************************************************************
@@ -39,23 +37,26 @@
 /* The console -- I chose to keep the master and slave
  * (UNIX) file descriptors around in case they are needed for
- * ioctls later.  The pid is needed to detroy the xterm if needed.
+ * ioctls later.  The pid is needed to destroy the xterm on close
  */
 typedef struct _CONSOLE {